/**
 * Copyright (c) 2022 Yansen Zhang
 * wxcomponent is licensed under Mulan PSL v2.
 * You can use this software according to the terms and conditions of the Mulan PSL v2.
 * You may obtain a copy of Mulan PSL v2 at:
 *          http://license.coscl.org.cn/MulanPSL2
 * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
 * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
 * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
 * See the Mulan PSL v2 for more details.
**/

package authorization

import (
	"encoding/json"
	"errors"

	wxerr "gitee.com/yansen_zh/wxcomponent/errors"
	"gitee.com/yansen_zh/wxcomponent/utils/request"
)

// ComponentTokenParam 获取令牌参数
type ComponentTokenParam struct {
	ComponentAppId        string `json:"component_appid"`
	ComponentAppSecret    string `json:"component_appsecret"`
	ComponentVerifyTicket string `json:"component_verify_ticket"`
}

// ComponentTokenResult 获取令牌结果
type ComponentTokenResult struct {
	wxerr.Error
	ComponentAccessToken string `json:"component_access_token"`
	ExpiresIn            int    `json:"expires_in"`
}

const (
	apiComponentToken = "https://apies.weixin.qq.com/cgi-bin/component/api_component_token"
)

// GetComponentToken 获取令牌
func GetComponentToken(data ComponentTokenParam) (*ComponentTokenResult, error) {
	if data.ComponentAppId == "" {
		return nil, errors.New("获取令牌 第三方平台的 appid 不能为空")
	}

	if data.ComponentAppSecret == "" {
		return nil, errors.New("获取令牌 第三方平台的 appsecret 不能为空")
	}

	if data.ComponentVerifyTicket == "" {
		return nil, errors.New("获取令牌 推送 ticket 不能为空")
	}

	resp, e2 := request.PostJSON(apiComponentToken, nil, data)
	if e2 != nil {
		return nil, e2
	}

	result := ComponentTokenResult{}
	if err := json.Unmarshal(resp, &result); err != nil {
		return nil, err
	}

	return &result, nil
}
