/**
 * Copyright (c) 2022 Yansen Zhang
 * wxcomponent is licensed under Mulan PSL v2.
 * You can use this software according to the terms and conditions of the Mulan PSL v2.
 * You may obtain a copy of Mulan PSL v2 at:
 *          http://license.coscl.org.cn/MulanPSL2
 * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
 * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
 * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
 * See the Mulan PSL v2 for more details.
**/

package authorization

import (
	"encoding/json"
	"errors"

	wxerr "gitee.com/yansen_zh/wxcomponent/errors"
	"gitee.com/yansen_zh/wxcomponent/utils/request"
)

// PushTicketParam 启动票据推送服务参数
type PushTicketParam struct {
	ComponentAppId  string `json:"component_appid"`
	ComponentSecret string `json:"component_secret"`
}

// PushTicketResult 启动票据推送服务参数
type PushTicketResult struct {
	wxerr.Error
}

const (
	apiStartPushTicket = "https://apies.weixin.qq.com/cgi-bin/component/api_start_push_ticket"
)

// StartPushTicket 启动 ticket 推送服务
func StartPushTicket(data PushTicketParam) (*PushTicketResult, error) {
	if data.ComponentAppId == "" {
		return nil, errors.New("启动 ticket 推送服务 第三方平台的 appid 不能为空")
	}

	if data.ComponentSecret == "" {
		return nil, errors.New("启动 ticket 推送服务 第三方平台的 appsecret 不能为空")
	}

	resp, e2 := request.PostJSON(apiStartPushTicket, nil, data)
	if e2 != nil {
		return nil, e2
	}

	result := PushTicketResult{}
	if err := json.Unmarshal(resp, &result); err != nil {
		return nil, err
	}

	return &result, nil
}
